# If this failed, open ucgs_a_681238_sm0876/load.r and uncomment a line to build the package
source("ucgs_a_681238_sm0876/load.r", chdir = TRUE)
source("util_burst_binom.R")

#----------
# fitting piece-wise constant probability
#----------
# generating data
set.seed(123)
p <- rep(c(0.3,0.7,0.3), each=50)
n_data <- length(p)
N <- rep(500, n_data)
k <- rbinom(n_data, N, p)

# fixed regularization penalty
# l1-segmentation (fused lasso)
fused.lasso.result <- burst.binom(k, N, lambda = 200)
# l0-segmentation
l0.segmentation.result <- burst.binom(k, N, use.l0 = TRUE, lambda = 200)

# cross-validation
# l1-segmentation (fused lasso)
fused.lasso.cv.result <- burst.binom.cv(k, N)
# l0-segmentation
l0.segmentation.cv.result <- burst.binom.cv(k, N, use.l0 = TRUE)

# plot result
plot(k/N, main="piecewise constant", ylab = "p")
lines(p, col="black")
lines(fused.lasso.result$p.hat, col="blue")
lines(l0.segmentation.result$p.hat, col="red")
lines(fused.lasso.cv.result$burst.binom.rlt$p.hat,lty=2,col="blue")
lines(l0.segmentation.cv.result$burst.binom.rlt$p.hat,lty=2, col="red")
legend("topleft",
       c("true", "l1 seg", "l0 seg", "l1 seg cv", "l0 seg cv"), 
       lty=c(1,1,1,2,2), 
       col=c("black", "blue", "red", "blue", "red"))

#----------
# fitting piece-wise linear probability (on logit scale) using trend-filtering
#----------
# generating data
set.seed(123)
theta = c(1:100,99:1) / 100 - 0.5
p = 1 / (1 + exp(-theta))
n_data <- length(p)
N <- rep(500, n_data)
k <- rbinom(n_data, N, p)

# fixed regularization penalty
# trend-filtering
trend.filtering.result <- burst.binom(k, N, lambda = 200, ord=1)

# cross-valitaion
trend.filtering.cv.result <- burst.binom.cv(k, N, ord=1)

# plot result
plot(k/N, main="piecewise linear", ylab = "p")
lines(p, col="black")
lines(trend.filtering.result$p.hat, col="blue")
lines(trend.filtering.cv.result$burst.binom.rlt$p.hat, col="blue",lty=2)
legend("topleft",
       c("true", "trend-filtering", "trend-filtering cv"), 
       lty=c(1,1,2), 
       col=c("black", "blue","blue"))
