#ifndef FLSA_HPP__
#define FLSA_HPP__

#include <R.h>
#include <Rdefines.h>
#include <Rmath.h>
#include <R_ext/Print.h>
#include <R_ext/Random.h>


#include "util.hpp"

namespace FLSA {

typedef double RealType;

using namespace Util;

struct MsgElt {
  // the location of the knot
  RealType x_;
  // the sign variable that tells us
  // whether this was a left or right end-point of the
  // segment
  bool sgn_;
  // a delta which can be used to reconstruct the function
  // if we move from the first knot to the last or from
  // the last to the first
  RealType lin_;
  RealType quad_;
};

class Msg {
public:
  vector<MsgElt> buf_;
  int start_idx_;
  int len_;
  MsgElt init_knot_;
  MsgElt end_knot_;
};

double MaxMsg(const Msg& msg);
void InitMsg(Msg * msg, int init_sz, RealType lin, RealType quad, RealType lambda2);
void ShiftMsg(Msg * msg, int check_freq);

double MaxMsg(const Msg& msg, double * max_val);

void BackTrace(double * x_hat, int seq_len, RealType * back_pointers, RealType last_msg_max);

int NumSegments(int seq_len, RealType * back_pointers, RealType last_msg_max);

void SegmentedBackTrace(double * x_hat, int * seg_bds, int seq_len, RealType * back_pointers,
                        RealType last_msg_max, int n_segs);
void SoftThresh(double * x, int len, double thresh);


}  // namespace FLSA


#endif
