FlsaDp = function(x, lambda2 = 1.0, lambda1 = 0.0,
                  initBufSz = 5000,
                  obsWts = NULL) {
  # To guarantee O(N) worst case running time, we estimate that initBufSz must
  # be set to something like 16 * length(x).  An implementation with an
  # initBufSz increasing buffer would require no up-front allocation, but
  # this has not been implemented yet.
  y = vector(mode = "numeric", length = length(x))

  if (!is.double(x)) {
    x = as.double(x)
  }

  .Call("FLSAOpt_RI", x, lambda2, lambda1, y, initBufSz, NULL)
  return(y)
}

IsotonicDp = function(x, lambda2 = 1.0,
                      initBufSz = 5000, segmented = TRUE,
                      obsWts = NULL) {
  if (segmented) {
    y = vector(mode = "list", length = 2)
  } else {
    y = vector(mode = "numeric", length = length(x))
  }


  if (is.null(obsWts)) {
    obsWts = rep(1, length(x))
  }

  if (!is.double(x)) {
    x = as.double(x)
  }

  .Call("IsotonicDpOpt_RI", x, lambda2, y, initBufSz, obsWts)
  return(y)
}

UnimodalDp = function(x, lambda2 = 1e-5, initBufSz = 5000, obsWts = NULL) {
  y = rep(-100, length(x))

  if (is.null(obsWts)) {
    obsWts = rep(1, length(x))
  }

  if (!is.double(x)) {
    x = as.double(x)
  }


  retHeights = rep(Inf, length(x))
  retCrit = rep(Inf, length(x))

  .Call("UnimodalDpOpt_RI", x, lambda2, retCrit, retHeights,
        y, initBufSz, obsWts)

  retCrit = retCrit + -0.5 * sum(x^2)
  return(list(crit = retCrit, height = retHeights, fit = y))
}
