# to compile the C code call 'FLBuildLib' defined below

LoadPackages = function() {
  libNames = library()
  
  desiredLibs = c("flsa", "Iso")
  
  for (lib in desiredLibs) {
    if (lib %in% libNames$results[,1]) {
      require(lib, character.only = TRUE)
    } else {
      print(sprintf("tried to load package %s but failed", lib))
    }
  }
}


SimSegs = function(len, nSegs = 3, segVars = 4, noiseVar = 1) {
  eps = rnorm(len, 0, sqrt(noiseVar))
  
  jumpIdx = sample(1:len, nSegs - 1)
  
  segInds = rep(0, len)
  segInds[jumpIdx] = 1
  segInds = 1 + cumsum(segInds)
  
  x = rep(0, len)
  segMeans = rnorm(nSegs, 0, sqrt(segVars))
  for (segIdx in 1:nSegs) {
    x[segInds == segIdx] = segMeans[segIdx]
  }
  
  y = (x + eps)

  optimalEsts = rep(0, len)
  segMeans = rnorm(nSegs, 0, sqrt(segVars))
  for (segIdx in 1:nSegs) {
    idx = segInds == segIdx
    optimalEsts[idx] = mean(y[idx])
  }
  return(list(x = y, jumpIdx = jumpIdx, segInds = segInds,
              segMeans = segMeans, optimalEsts = optimalEsts))
}

RepEach = function(x, n) {
  n = as.integer(n)
  return(.Call("RepEach", x, n))
}

ArrayIdxFromLinIdx = function(inds, arrayDim) {
  r = matrix(nrow = length(inds), ncol = length(arrayDim), data = as.integer(0))
  
  inds = inds - as.integer(1)
  
  for (k in 1:length(arrayDim)) {
    r[,k] = 1 + (inds %% arrayDim[k])
    inds = as.integer(inds / arrayDim[k])
  }
  return(r)
}

FLObjective = function(b, o, lambda1 = 0, lambda2 = 0) {
  d = length(b)

  r1 = sum( abs(b) * lambda1 )
  b1 = b[-d]
  b2 = b[-1]
  r1 = r1 + sum( abs(b2-b1)*lambda2 )

  i1 = which(!is.na(o))
  if(length(i1) > 0){
    r1 = r1 + 0.5*sum( (o[i1]-b[i1])^2 )
  }

  return(r1)
}

ClearBuildFiles = function() {
  system("rm c_src/*.o")
}

IsWindows = function() {
  return(.Platform$OS.type == "windows")
}

FileExists = function(fnm) {
  return(!is.na(file.info(fnm)$size))
}

FLLibPath = function() {
  libExt = ifelse(IsWindows(), ".dll", ".so")
  libFname = paste("lib/FlDp", libExt, sep="")
  return(libFname)
}

FLUnloadLib = function() {
  if (is.loaded("FLSAOpt_RI")) {
    dyn.unload(FLLibPath())
  }
}

FLLoadLib = function() {
  FLUnloadLib()
  if (FileExists(FLLibPath())) {
    dyn.load(FLLibPath())
  } else {
    stop(sprintf("could not find library at path : %s", FLLibPath()))
  } 
}

FLBuildLib = function() {
  # Loads / Builds the C code used in these functions
  libFname = FLLibPath() 
  FLUnloadLib()
                  
  srcFiles = c( "c/L0ExpFam.cpp",
                "c/L0Dp.cpp",
                "c/util.cpp",
                "c/flsa.cpp",
                "c/isotonic.cpp" )
  
  buildCmd <- paste("R CMD SHLIB -o ", libFname, sep="")
  
  for(j in 1:length(srcFiles)){
    buildCmd <- paste(buildCmd, srcFiles[j], sep=" ")   
  }  
  system(buildCmd)
}
